unit MyRegistry;

interface

uses Windows, SysUtils, Classes, Registry;

type
  TMyRegistry = class(TRegistry)
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  function OpenKeyRO(const Key: string; CanCreate: Boolean): Boolean;
 end;

 xBufType = array [0..$FFFF] of char;

const MAXRESENTRYS = 63;

const InterfaceType: array [0..15] of string =
                       ('Internal',
                        'Isa',
                        'Eisa',
                        'MicroChannel',
                        'TurboChannel',
                        'PCIBus',
                        'VMEBus',
                        'NuBus',
                        'PCMCIABus',
                        'CBus',
                        'MPIBus',
                        'MPSABus',
                        'ProcessorInternal',
                        'InternalPowerBus',
                        'PNPISABus',
                        'MaximumInterfaceType');

 type  TResourceType = (tNull,tPort,tInterrupt,tMemory,tDMA,tDeviceSpecific);
       TPartialResourceDescriptor = packed record
         typ   : Byte;
         share : Byte;
         flags : word;
         case TResourceType of
            tPort,tMemory    : (Address:int64;Length:integer);
            tInterrupt       : (Level,Vector,Affinity:integer);
            tDMA             : (Channel,Port,Reserved:integer);
            tDeviceSpecific  : (DataSize,Reserved0,Reserved1:integer);
        end;


 type  PPartialResourceList = ^TPartialResourceList;
       TPartialResourceList = record
         header : packed record
              InterfaceType : dword;
              BusNumber     : dword;
              Version       : word;
              Revision      : word;
              Count         : dword;
            end;
         Entrys : array [0..MAXRESENTRYS] of TPartialResourceDescriptor;
       end;

function CheckValue(RKey:HKEY;Key,Value,cValue:string):boolean;
function ReadValue(RKey:HKEY;Key,Value:string):string;
function ReadValueBuf(RKey:HKEY;Key,Value:string):xBufType;
procedure ReadKeys(RKey:HKEY;Key:string; var erg:TStringList);
procedure ReadValues(RKey:HKEY;Key:string; var erg:TStringList);

Const HKLM=HKEY_LOCAL_MACHINE;
      HKCU=HKEY_CURRENT_USER;
      HKCR=HKEY_CLASSES_ROOT;
      HKCC=HKEY_CURRENT_CONFIG;


var LastRead_CountBytes:integer;

implementation

function IsRelative(const Value: string): Boolean;
begin
  Result := not ((Value <> '') and (Value[1] = '\'));
end;

function TMyRegistry.OpenKeyRO(const Key: string; CanCreate: Boolean): Boolean;
var
  TempKey: HKey;
  S: string;
  Disposition: Integer;
  Relative: Boolean;
begin
  S := Key;
  Relative := IsRelative(S);
  if not Relative then Delete(S, 1, 1);
  TempKey := 0;
  if not CanCreate or (S = '') then
  begin
    Result := RegOpenKeyEx(GetBaseKey(Relative), PChar(S), 0,
      KEY_ALL_ACCESS, TempKey) = ERROR_SUCCESS;
  end else
    Result := RegCreateKeyEx(GetBaseKey(Relative), PChar(S), 0, nil,
      REG_OPTION_NON_VOLATILE, KEY_READ, nil, TempKey, @Disposition) = ERROR_SUCCESS;
  if Result then
  begin
    if (CurrentKey <> 0) and Relative then S := CurrentPath + '\' + S;
    ChangeKey(TempKey, S);
  end;
end;

function CheckValue(RKey:HKEY;Key,Value,cValue:string):boolean;
var mReg:TMyRegistry;
    dt:TRegDataType;
begin
 mReg:=TMyRegistry.Create;
 with mReg do
  begin
   RootKey:=RKey;
   if OpenKeyRO(Key,false) then
    begin
     dt:=GetDataType(Value);
     try
      case dt of
        rdString,
        rdExpandString : Result:=(ReadString(Value)=cValue);
        rdInteger      : Result:=(ReadInteger(Value)=StrToInt(cValue));
       else Result:=(ReadString(Value)=cValue);
      end;
     except
      on ERegistryException do Result:=false;
     end;
    end else Result:=false;
  end;
 mReg.Free;
end;

function ReadValue(RKey:HKEY;Key,Value:string):string;
var mReg:TMyRegistry;
    dt:TRegDataType;
    bf: array [0..255] of char;
begin
 FillChar(bf,sizeof(bf),#0);
 mReg:=TMyRegistry.Create;
 with mReg do
  begin
   RootKey:=RKey;
   if OpenKeyRO(Key,false) then
    begin
     dt:=GetDataType(Value);
     try
      case dt of
        rdUnknown      : begin
                           ReadBinaryData(Value,bf,255);
                           Result:=bf;
                         end;
        rdString,
        rdExpandString : Result:=ReadString(Value);
        rdInteger      : Result:=IntToStr(ReadInteger(Value));
       else Result:='';
      end;
     except
      on ERegistryException do Result:='';
     end;
    end else Result:='';
  end;
 mReg.Free;
end;

function ReadValueBuf(RKey:HKEY;Key,Value:string):xBufType;
var mReg:TMyRegistry;
    dt:TRegDataType;
    bf: xBufType;
begin
 FillChar(bf,sizeof(bf),#0);
 LastRead_CountBytes:=0;
 mReg:=TMyRegistry.Create;
 with mReg do
  begin
   RootKey:=RKey;
   if OpenKeyRO(Key,false) then
    begin
     dt:=GetDataType(Value);
     try
      case dt of
        rdUnknown      : begin
                           LastRead_CountBytes:=ReadBinaryData(Value,bf,$FFFF);
                           Result:=bf;
                         end;
        rdString,
        rdExpandString : begin
                          LastRead_CountBytes:=ReadBinaryData(Value,bf,$FFFF);
                          Result:=bf;
                         end;
        rdInteger      : begin
                          LastRead_CountBytes:=ReadBinaryData(Value,bf,$FFFF);
                          Result:=bf;
                         end;
       else Result:='';
      end;
     except
      on ERegistryException do Result:='';
     end;
    end else Result:='';
  end;
 mReg.Free;
end;

procedure ReadKeys(RKey:HKEY;Key:string; var erg:TStringList);
var mReg:TMyRegistry;
//    dt:TRegDataType;
//    bf: array [0..255] of char;
begin
 mReg:=TMyRegistry.Create;
 with mReg do
  begin
   RootKey:=RKey;
   if OpenKeyRO(Key,false) then
    begin
     GetKeyNames(erg);
    end;
  end;
 mReg.Free;
end;

procedure ReadValues(RKey:HKEY;Key:string; var erg:TStringList);
var mReg:TMyRegistry;
//    dt:TRegDataType;
//    bf: array [0..255] of char;
begin
 mReg:=TMyRegistry.Create;
 with mReg do
  begin
   RootKey:=RKey;
   if OpenKeyRO(Key,false) then
    begin
     GetValueNames(erg);
    end;
  end;
 mReg.Free;
end;

end.
