unit driver;

interface

uses services;

function InstallDriver(DriverName,DriverExec:string):boolean;
function RemoveDriver(DriverName:string):boolean;
function StartDriver(DriverName:string):boolean;
function StopDriver(DriverName:string):boolean;
function TestDriver(DriverName:string):boolean;
function GetDriverStatus(DriverName:string):integer;
function GetDriverConfig(DriverName:string):PQueryServiceConfig;

const ShowErrorBoxes:boolean=true;

var DriverLastError,DriverLastErrorMsg:string;

implementation

uses windows,forms,sysutils;


function WinErrorAsString(id : INTEGER) : STRING;
var
  a   : ARRAY [0..$FFFF] OF CHAR;
begin
  FillChar(a, SizeOf(a), #0);
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nil, id, LANG_SYSTEM_DEFAULT, @a, MAX_PATH, nil);
  Result := a;
  while Pos(#13, Result)>0 do Delete(Result, Pos(#13, Result), 1);
  while Pos(#10, Result)>0 do Delete(Result, Pos(#10, Result), 1);
end;

procedure FatalWinError(msg : STRING);
begin
  DriverLastError:=WinErrorAsString(GetLastError);
  DriverLastErrorMsg:=msg;
  if ShowErrorBoxes then
    Application.MessageBox(PChar(DriverLastError),
                           PChar('DriverControl: '+msg),MB_ICONSTOP + MB_OK);
end;

function OpenManager:TSCHandle;
begin
  Result:=OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if Result=0 then FatalWinError('Cant open Service Control Manager !');
end;

function InstallDriver(DriverName,DriverExec:string):boolean;
var hSCM:TSCHandle;
    hService:TSCHandle;
begin
Result:=false;
hSCM:=OpenManager;
if hSCM<>0 then
  begin
   hService:=CreateService(hSCM, PChar(DriverName), PChar(DriverName),
                           SERVICE_ALL_ACCESS,  SERVICE_KERNEL_DRIVER,
                           SERVICE_DEMAND_START,SERVICE_ERROR_NORMAL,
                           PChar(DriverExec), nil, nil, nil, nil, nil);
  if hService = 0 then
   begin
    CloseServiceHandle(hSCM);
    FatalWinError('Cant install '+DriverName+' !');
    exit;
   end else Result:=true;
  CloseServiceHandle(hSCM);
  CloseServiceHandle(hService);
  end;
end;

function RemoveDriver(DriverName:string):boolean;
var hSCM:TSCHandle;
    hService:TSCHandle;
begin
Result:=false;
hSCM:=OpenManager;
if hSCM<>0 then
 begin
  hService:=OpenService(hSCM, PChar(DriverName), SERVICE_ALL_ACCESS);
  if hService = 0 then
   begin
    CloseServiceHandle(hSCM);
    FatalWinError('Cant open '+DriverName+' !');
    exit;
   end else Result:=DeleteService(hService);
  CloseServiceHandle(hSCM);
  CloseServiceHandle(hService);
 end;
end;

function StartDriver(DriverName:string):boolean;
var hSCM:TSCHandle;
    hService:TSCHandle;
begin
Result:=false;
hSCM:=OpenManager;
if hSCM<>0 then
 begin
  hService:=OpenService(hSCM, PChar(DriverName), SERVICE_ALL_ACCESS);
  if hService = 0 then
   begin
    CloseServiceHandle(hSCM);
    FatalWinError('Cant open '+DriverName+' !');
    exit;
   end else Result:=StartService(hService,0,NIL);
   if not Result then FatalWinError('Cant start '+DriverName+' !');
  CloseServiceHandle(hSCM);
  CloseServiceHandle(hService);
 end;
end;

function StopDriver(DriverName:string):boolean;
var hSCM:TSCHandle;
    hService:TSCHandle;
    ServiceStatus:TServiceStatus;
begin
Result:=false;
hSCM:=OpenManager;
if hSCM<>0 then
 begin
  hService:=OpenService(hSCM, PChar(DriverName), SERVICE_ALL_ACCESS);
  if hService = 0 then
   begin
    CloseServiceHandle(hSCM);
    FatalWinError('Cant open '+DriverName+' !');
    exit;
   end else Result:=ControlService(hService,SERVICE_CONTROL_STOP,
                                   ServiceStatus);
  if not Result then FatalWinError('Cant stop '+DriverName+' !');
  CloseServiceHandle(hSCM);
  CloseServiceHandle(hService);
 end;
end;

function TestDriver(DriverName:string):boolean;
var hSCM:TSCHandle;
    hService:TSCHandle;
begin
Result:=false;
hSCM:=OpenManager;
if hSCM<>0 then
 begin
  hService:=OpenService(hSCM, PChar(DriverName), SERVICE_ALL_ACCESS);
  if hService <> 0 then
   begin
    CloseServiceHandle(hService);
    Result:=true;
   end;
  CloseServiceHandle(hSCM);
 end;
end;

function GetDriverStatus(DriverName:string):integer;
var hSCM:TSCHandle;
    hService:TSCHandle;
    mStatus:TServiceStatus;

begin
Result:=-1;
hSCM:=OpenManager;
if hSCM<>0 then
 begin
  hService:=OpenService(hSCM, PChar(DriverName), SERVICE_ALL_ACCESS);
  if hService <> 0 then
   begin
    if QueryServiceStatus(hService,mStatus) then
     begin
      Result:=mStatus.dwCurrentState;
     end;
    CloseServiceHandle(hService);
   end;
  CloseServiceHandle(hSCM);
 end;
end;

function GetDriverConfig(DriverName:string):PQueryServiceConfig;
var hSCM:TSCHandle;
    hService:TSCHandle;
    BytesNeeded:integer;
    buffer:pointer;

begin
GetMem(Buffer,4096);
PQueryServiceConfig(Buffer).dwServiceType:=-1;
hSCM:=OpenManager;
if hSCM<>0 then
 begin
  hService:=OpenService(hSCM, PChar(DriverName), SERVICE_ALL_ACCESS);
  if hService <> 0 then
   begin
    if not QueryServiceConfig(hService,Buffer,4096,BytesNeeded) then
      begin
       FatalWinError('Cant query driver config ! ');
      end;
    CloseServiceHandle(hService);
   end;
  CloseServiceHandle(hSCM);
 end;
Result:=buffer;
end;

end.
