//
// Unit UniPort
//
// Beim Initialisieren der Unit wird der Treiber UniPort
// geladen und UniHandle wird durch den Aufruf von
// CreateFile initialisiert. Auerdem wird eine Exit Routine
// registriert, die beim Beenden des Programms das Handle
// UniHandle schliesst und den Treiber beendet und entlaedt.
// Der Zugriff auf die Ports erfolgt mit den Funktionen
// ReadPortXX und WritePortXX.
// Der Recordtyp ParamBlockStruct ist eine Umsetzung der
// mParamBlock Struktur, die im Treiber definiert wurde.
// Da Delphi das Macro CTL_CODE nicht untertstuetzt, muessen
// die I/O Control Codes (IOCTL_XXX) von Hand deklariert
// werden.

unit UniPort;

interface

uses Windows, SysUtils, driver;

function ReadPortB(Address:integer):byte;
function ReadPortW(Address:integer):word;
function ReadPortDW(Address:integer):DWORD;
procedure WritePortB(Address:integer;Value:byte);
procedure WritePortW(Address:integer;Value:word);
procedure WritePortDW(Address:integer;Value:dword);

implementation

type ParamBlockStruct = packed record
               PortAdr  :  integer;
               Size     :  integer;
               Value    :  integer;
              end;

const IOCTL_UniPort_ReadPort  = $222004;
      IOCTL_UniPort_WritePort = $222008;

var OldExitProc:Pointer;
    UniHandle:THandle;

//
// InitPorts wird beim Initialisieren der Unit aufgerufen
//
function InitPorts:boolean;
var sa:PSecurityAttributes;
begin
InstallDriver('uniport',ExtractFilePath(Paramstr(0))+'uniport.sys');
StartDriver('uniport');
sa:=new(PSecurityAttributes);
sa.nLength:=sizeof(TSecurityAttributes);
sa.lpSecurityDescriptor:=NIL;
sa.bInheritHandle:=false;
UniHandle:=CreateFile('\\.\uniport',GENERIC_READ,FILE_SHARE_READ,
                     sa,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
Result:=(INVALID_HANDLE_VALUE<>UniHandle);
end;

function ReadPortB(Address:integer):byte;
var mParamBlock:ParamBlockStruct;
    x:integer;
    rb:cardinal;
begin
 mParamBlock.PortAdr:=Address;
 mParamBlock.Size:=1;
 if DeviceIoControl(UniHandle,IOCTL_UniPort_ReadPort,@mParamBlock,
                 SizeOf(mParamBlock),@x,sizeof(x),rb,NIL) then
     Result:=x else Result:=0;
end;

function ReadPortW(Address:integer):word;
var mParamBlock:ParamBlockStruct;
    x:integer;
    rb:cardinal;
begin
 mParamBlock.PortAdr:=Address;
 mParamBlock.Size:=2;
 if DeviceIoControl(UniHandle,IOCTL_UniPort_ReadPort,@mParamBlock,
                 SizeOf(mParamBlock),@x,sizeof(x),rb,NIL) then
     Result:=x else Result:=0;
end;

function ReadPortDW(Address:integer):dword;
var mParamBlock:ParamBlockStruct;
    x:integer;
    rb:cardinal;
begin
 mParamBlock.PortAdr:=Address;
 mParamBlock.Size:=4;
 if DeviceIoControl(UniHandle,IOCTL_UniPort_ReadPort,@mParamBlock,
                 SizeOf(mParamBlock),@x,sizeof(x),rb,NIL) then
     Result:=x else Result:=0;
end;

procedure WritePortB(Address:integer;Value:byte);
var mParamBlock:ParamBlockStruct;
    rb:cardinal;
begin
 mParamBlock.PortAdr:=Address;
 mParamBlock.Size:=1;
 mParamBlock.Value:=Value;
 DeviceIoControl(UniHandle,IOCTL_UniPort_WritePort,@mParamBlock,
                 SizeOf(mParamBlock),NIL,0,rb,NIL);
end;

procedure WritePortW(Address:integer;Value:word);
var mParamBlock:ParamBlockStruct;
    rb:cardinal;
begin
 mParamBlock.PortAdr:=Address;
 mParamBlock.Size:=2;
 mParamBlock.Value:=Value;
 DeviceIoControl(UniHandle,IOCTL_UniPort_WritePort,@mParamBlock,
                 SizeOf(mParamBlock),NIL,0,rb,NIL);
end;

procedure WritePortDW(Address:integer;Value:dword);
var mParamBlock:ParamBlockStruct;
    rb:cardinal;
begin
 mParamBlock.PortAdr:=Address;
 mParamBlock.Size:=4;
 mParamBlock.Value:=Value;
 DeviceIoControl(UniHandle,IOCTL_UniPort_WritePort,@mParamBlock,
                 SizeOf(mParamBlock),NIL,0,rb,NIL);
end;

//
// Unload wird beim Beenden des Programms aufgerufen
//
procedure Unload;
begin
CloseHandle(UniHandle);
ExitProc:=OldExitProc;
StopDriver('uniport');
RemoveDriver('uniport');
end;

//
// der Initialisierungscode
//
begin
if InitPorts then
 begin
  OldExitProc:=ExitProc;
  ExitProc:=@Unload;
 end;
end.
